/*
 * Copyright 2001 OpenPrivacy.org.  All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the LICENSE which you should have obtaind with this package. 
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.
 */

package talon.components;

import java.util.*;

import talon.Component;


/**
 * Logger interface which can be used within an application for debug and
 * runtime information.
 *
 * Should support the following log levels
 * <pre>
 * 
 * debug 
 * info
 * error
 * warn
 * fatal
 *
 * </pre>
 * 
 * Also each level should support the following options:
 *
 * <pre>
 * 
 * message (required)
 * channel (optional)
 * throwable (optional)
 * 
 * </pre>
 *
 * @author <a href="mailto:burton@relativity.yi.org">Kevin A. Burton</a>
 * @version $Id: Logger.java,v 1.6 2001/08/02 07:13:03 burton Exp $
 */
public interface Logger extends Component {


    /**
     * Log a message.
     *
     * @author <a href="mailto:burton@relativity.yi.org">Kevin A. Burton</a>
     * @version $Id: Logger.java,v 1.6 2001/08/02 07:13:03 burton Exp $
     */
    public void message( Object message );

    /**
     * Log a message.
     *
     * @param category The category object.  The classname of this object will be
     *               entered into the log
     * @author <a href="mailto:burton@relativity.yi.org">Kevin A. Burton</a>
     * @version $Id: Logger.java,v 1.6 2001/08/02 07:13:03 burton Exp $
     */
    public void message( Object message, Object category );
    
    /**
     * Log a message.
     *
     * @author <a href="mailto:burton@relativity.yi.org">Kevin A. Burton</a>
     * @version $Id: Logger.java,v 1.6 2001/08/02 07:13:03 burton Exp $
     */
    public void debug( Object message );

    /**
     * Log a debug message.
     *
     * @param category The category object.  The classname of this object will be
     *               entered into the log
     * @author <a href="mailto:burton@relativity.yi.org">Kevin A. Burton</a>
     * @version $Id: Logger.java,v 1.6 2001/08/02 07:13:03 burton Exp $
     */
    public void debug( Object message, Object category );

    
    /**
     * Log a warning message.
     *
     * @author <a href="mailto:burton@relativity.yi.org">Kevin A. Burton</a>
     * @version $Id: Logger.java,v 1.6 2001/08/02 07:13:03 burton Exp $
     */
    public void warn( Object message );
    
    /**
     * Log a warning message.
     *
     * @param category The category object.  The classname of this object will be
     *               entered into the log
     * @author <a href="mailto:burton@relativity.yi.org">Kevin A. Burton</a>
     * @version $Id: Logger.java,v 1.6 2001/08/02 07:13:03 burton Exp $
     */
    public void warn( Object message, Object category );
    
    /**
     * Log an error message.
     *
     * @author <a href="mailto:burton@relativity.yi.org">Kevin A. Burton</a>
     * @version $Id: Logger.java,v 1.6 2001/08/02 07:13:03 burton Exp $
     */
    public void error( Object message );

    /**
     * Log an error message.
     *
     * @author <a href="mailto:burton@relativity.yi.org">Kevin A. Burton</a>
     * @version $Id: Logger.java,v 1.6 2001/08/02 07:13:03 burton Exp $
     */
    public void error( Object message, Throwable throwable );
    
    /**
     * Log an Exception.
     *
     * @author <a href="mailto:burton@relativity.yi.org">Kevin A. Burton</a>
     * @version $Id: Logger.java,v 1.6 2001/08/02 07:13:03 burton Exp $
     */
    public void error( Object message, Object category, Throwable t );

    /**
     * Get a new logger that hosts the given category.
     *
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: Logger.java,v 1.6 2001/08/02 07:13:03 burton Exp $
     */
    public Logger getLogger( Class category );
        
    /**
     * @see #getLogger
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: Logger.java,v 1.6 2001/08/02 07:13:03 burton Exp $
     */
    public Logger getLogger( Object object );
    
}
