/*
 * Copyright 2001 OpenPrivacy.org.  All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the LICENSE which you should have obtaind with this package. 
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.
 */

package talon.implementations;

//standard Java stuff
import java.io.*;
import java.util.*;

import talon.*;
import talon.components.*;
import talon.util.*;


/**
 * Used when working with property files.
 *
 * <p>
 * This object depends on the following initialization properties:
 *
 * <dl>
 *   <dt>project
 *     <dd>Name of the project this property file is going to be used in.
 *         This is used so that we can find a [PROJECT].properties file
 *         in user.home, java.home or /etc.
 * </dl>
 * <p>
 * Generally this object should be deployed as a Singleton... maybe a Named
 * Singleton
 * 
 * @author <a href="mailto:burton@relativity.yi.org">Kevin A. Burton</a>
 * @version $Id: FilePropertyManager.java,v 1.8 2001/06/11 08:58:30 burton Exp $
 */
public class FilePropertyManager extends BasePropertyManager {

    public static final String[] REQUIRED_PROPERTIES = { "file" };
    
    private String propertyFilename = null;
    
    /**
     * Constructor when used as a Component
     *
     * @author <a href="mailto:burton@relativity.yi.org">Kevin A. Burton</a>
     * @version $Id: FilePropertyManager.java,v 1.8 2001/06/11 08:58:30 burton Exp $
     */
    public FilePropertyManager( ) { } 
    

    /**
     * Perform any necessary initialization
     *
     * @see Component
     * @author <a href="mailto:burton@relativity.yi.org">Kevin A. Burton</a>
     * @version $Id: FilePropertyManager.java,v 1.8 2001/06/11 08:58:30 burton Exp $
     */
    public void init() throws TalonException {
           
        try {

            Properties props = new Properties();
            props.load( new FileInputStream( getPropertyFilename() ) );

            this.setProperties( props );
            
        } catch ( Exception e ) {
            e.printStackTrace();
            throw new TalonException( e.getMessage() );
        }

    }


    /**
     * Get the property file that this was initialized from. 
     *
     * @author <a href="mailto:burton@relativity.yi.org">Kevin A. Burton</a>
     * @version $Id: FilePropertyManager.java,v 1.8 2001/06/11 08:58:30 burton Exp $
     */
    public String getPropertyFilename() {
        return this.propertyFilename;
    }
    
}
