/*
 * Copyright 2001 OpenPrivacy.org.  All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the LICENSE which you should have obtaind with this package. 
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.
 */

package talon.util;

import java.io.*;
import java.net.*;
import java.util.*;

import talon.*;
import talon.components.*;
import talon.interfaces.*;
import talon.implementations.*;
import talon.util.net.*;
import talon.resources.*;

/**
 * Compute the Runtime of an application.  This class will keep track the number
 * of seconds since it was instantiated.
 *
 * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
 * @version $Id: RuntimeUtil.java,v 1.5 2001/12/02 05:37:48 burton Exp $
 */
public class RuntimeUtil {

    long epoch = 0;
    
    public RuntimeUtil() {

        this.epoch = System.currentTimeMillis();
        
    }

    public long getElapsed() {

        return System.currentTimeMillis() - epoch;
        
    }

    public long getElapsedSeconds() {

        return (long)( System.currentTimeMillis() - epoch ) / 1000;
        
    }

    /**
     * Output the number of seconds since epoch.
     *
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: RuntimeUtil.java,v 1.5 2001/12/02 05:37:48 burton Exp $
     */
    public void output() {

        Log.message( "Runtime info: Elapsed time: " + getElapsedSeconds() +
                     " seconds ( or " + getElapsed() + " milliseconds ) " );

        Log.message( "Runtime info: Free memory (in bytes): " + Runtime.getRuntime().freeMemory() );

        Log.message( "Runtime info: Total memory (in bytes): " + Runtime.getRuntime().totalMemory() );
        
    }

    /**
     * Output to stdout.
     *
     * @author <a href="mailto:burton@relativity.yi.org">Kevin A. Burton</a>
     */
    public void stdout() {

        System.out.println( "Runtime info: Elapsed time: " + getElapsedSeconds() +
                            " seconds ( or " + getElapsed() + " milliseconds ) " );
        
        System.out.println( "Runtime info: Free memory (in bytes): " + Runtime.getRuntime().freeMemory() );
        
        System.out.println( "Runtime info: Total memory (in bytes): " + Runtime.getRuntime().totalMemory() );

    }
    
}
