/*
 * Copyright 2001 OpenPrivacy.org.  All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the LICENSE which you should have obtaind with this package. 
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.
 */

package talon.util.net;

import java.net.*;
import java.io.*;
import java.util.*;
import talon.util.net.*;
import talon.interfaces.*;
import talon.*;


/**
 * Basically a file:// URL.  The main advantage here is that you can point right
 * to a file (AKA java.io.File) and this class will:
 *
 * - Make sure that file exists or throw an exception.
 *
 * - Add the necessary file:// portion of the URI.
 * 
 * @author <a href="mailto:burton@openprivacy.org">Kevin A. Burton</a>
 * @version $Id: FileURL.java,v 1.4 2002/02/27 19:05:30 burton Exp $
 */
public class FileURL extends URI {

    private File file = null;
    
    /**
     * 
     *
     * @author <a href="mailto:burton@openprivacy.org">Kevin A. Burton</a>
     * @version $Id: FileURL.java,v 1.4 2002/02/27 19:05:30 burton Exp $
     */
    public FileURL( File file ) {

        super( "file://" + file.getAbsolutePath() );

        this.file = file;
    }

    /**
     * Shortcut for using strings.
     *
     * @author <a href="mailto:burton@openprivacy.org">Kevin A. Burton</a>
     * @version $Id: FileURL.java,v 1.4 2002/02/27 19:05:30 burton Exp $
     */
    public FileURL( String file ) {

        this( new File( file ) );

    }

    /**
     * Require that this file exists or throw an IOException
     *
     * @author <a href="mailto:burton@openprivacy.org">Kevin A. Burton</a>
     * @version $Id: FileURL.java,v 1.4 2002/02/27 19:05:30 burton Exp $
     */
    public void require() throws IOException {

        if ( ! file.exists() ) {

            throw new IOException( "File does not exist: " + toString() );
            
        } 
        
    }

    /**
     * Convert the given File URL to a File.
     *
     * @author <a href="mailto:burton@openprivacy.org">Kevin A. Burton</a>
     */
    public static File toFile( String location ) {

        if ( location.startsWith( "file:///" ) ) {

            return new File( location.substring( "file://".length(), location.length() ) );
            
        } else if ( location.startsWith( "file://" ) ) {
            
            return new File( location.substring( "file:/".length(), location.length() ) );
            
        } else if ( location.startsWith( "file:/" ) ) {
            
            return new File( location.substring( "file:".length(), location.length() ) );
            
        } else {

            return null;
            
        }
        
    }
    
}
