/*
 * Copyright 2001 OpenPrivacy.org.  All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the LICENSE which you should have obtaind with this package. 
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.
 */

package talon.util.net;

import java.net.*;
import java.io.*;

/**
 * First class object for the W3C Schema concept of a uriReference.  
 *
 * TODO: use a regexp on the reference to verify it is a valid URI.
 *
 * @author <a href="mailto:burton@relativity.yi.org">Kevin A. Burton</a>
 * @version $Id: URI.java,v 1.10 2001/12/17 23:55:33 burton Exp $
 */
public class URI {

    private String reference = "";
    private boolean isURL = false;

    /**
     * Noop constructor if called as a bean.
     *
     * @author <a href="mailto:burton@relativity.yi.org">Kevin A. Burton</a>
     * @version $Id: URI.java,v 1.10 2001/12/17 23:55:33 burton Exp $
     */
    public URI() { }

    /**
     * 
     *
     * @author <a href="mailto:burton@relativity.yi.org">Kevin A. Burton</a>
     * @version $Id: URI.java,v 1.10 2001/12/17 23:55:33 burton Exp $
     */
    public URI( String reference ) {
        this.reference = reference;
    }

    /**
     *
     *
     * @author <a href="mailto:burton@relativity.yi.org">Kevin A. Burton</a>
     * @version $Id: URI.java,v 1.10 2001/12/17 23:55:33 burton Exp $
     */
    public URI( String reference,
                boolean isURL ) {

        this.reference = reference;
        this.isURL = isURL;

    }
    
    /**
     * @see Object.toString#
     * @author <a href="mailto:burton@relativity.yi.org">Kevin A. Burton</a>
     * @version $Id: URI.java,v 1.10 2001/12/17 23:55:33 burton Exp $
     */
    public String toString() {
        return this.reference;
    }

    /**
     * Return true if this URL is network locatable.
     *
     * @author <a href="mailto:burton@relativity.yi.org">Kevin A. Burton</a>
     * @version $Id: URI.java,v 1.10 2001/12/17 23:55:33 burton Exp $
     */
    public boolean isURL() {
        return this.isURL;
    }

    /**
     *
     *
     * @author <a href="mailto:burton@relativity.yi.org">Kevin A. Burton</a>
     * @version $Id: URI.java,v 1.10 2001/12/17 23:55:33 burton Exp $
     */
    public String getReference() { 
        return this.reference;
    }

    /**
     *
     *
     * @author <a href="mailto:burton@relativity.yi.org">Kevin A. Burton</a>
     * @version $Id: URI.java,v 1.10 2001/12/17 23:55:33 burton Exp $
     */
    public void setReference( String reference ) { 
        this.reference = reference;
    }

    /**
     * Return the scheme of this URL.  This is the portion of the URL before any
     * addressing portion.  This is usually expressed as
     * scheme://resource_location.  Example.  URI -> http://www.openprivacy.org
     * scheme ->  http
     *
     * @author <a href="mailto:burton@relativity.yi.org">Kevin A. Burton</a>
     * @version $Id: URI.java,v 1.10 2001/12/17 23:55:33 burton Exp $
     */
    public String getScheme() {

        String scheme = null;

        int index = this.reference.indexOf( ":" );

        if ( index != -1 ) {
            scheme = this.reference.substring( 0, index );
        } 
        
        return scheme;
        
    }
    
    /**
     * Get a URL representation of this URI.
     *
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: URI.java,v 1.10 2001/12/17 23:55:33 burton Exp $
     */
    public URL getURL() throws IOException {

        return new URL( this.toString() );
        
    }
    
    /**
     * Util method for getting a URI via a URI which is loaded as a resource.
     *
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: URI.java,v 1.10 2001/12/17 23:55:33 burton Exp $
     */
    public URL getResource() throws IOException {

        URL url = getClass().getResource( this.toString() );

        if ( url == null ) {
            
            throw new IOException( "Could not find URI as a resource:  " + this.toString() );
            
        } 

        System.out.println( "Found url: + " + url.toString() );
        
        return url;
        
    }

    /**
     * @see Object#hashCode
     * @author <a href="mailto:burton@relativity.yi.org">Kevin A. Burton</a>
     * @version $Id: URI.java,v 1.10 2001/12/17 23:55:33 burton Exp $
     */
    public boolean equals( Object obj ) {

        if ( obj instanceof java.lang.String ) {

            String location = (String)obj;
            
            return this.toString().equals( location );

        }

        if ( this.getClass() == obj.getClass() ) {

            URI other = (URI)obj;

            return toString().equals( other.toString() );

        }

        return false;

    }

    /**
     * @see Object#hashCode
     * @author <a href="mailto:burton@relativity.yi.org">Kevin A. Burton</a>
     * @version $Id: URI.java,v 1.10 2001/12/17 23:55:33 burton Exp $
     */
    public int hashCode() {

        return toString().hashCode();

    }
    
}

